'use strict';
const packageInfo = require('../package.json');
const queryString = require('querystring');

class baseProxy {
  isObject(value) {
    return value !== null && typeof value === 'object' && !Array.isArray(value);
  }
  transformObjectToArray(obj) {
    const result = [];
    for (const key in obj) {
      if (obj.hasOwnProperty(key)) {
        const context = key;
        const host = obj[key];
        result.push({ context, host });
      }
    }
    return result;
  }
  // check params
  checkOut(option) {
    const { name } = packageInfo;
    if (!option) {
      throw Error(`${name}: missing main parameters`);
    }
    if (!(option.proxies && (Array.isArray(option.proxies) || this.isObject(option.proxies)))) {
      throw Error(`${name}: The parameter is required and the type must be Array or Object: proxyArray`);
    }
    if (option.rewrite && typeof option.rewrite !== 'function') {
      throw Error(`${name}: the type must be Function: rewrite`);
    }
    // 校验全局参数
    this.options = {
      proxyTimeout: option.proxyTimeout || 30000,
      proxies: option.proxies,
      handleReq: option.proxyReq,
      handleRes: option.proxyRes,
      handleError: option.error,
    };
    // 判断全局rewrite
    if (option.rewrite === false) {
      this.options.rewrite = () => path => path.replace(/^\//, '');
    } else {
      this.options.rewrite = option.rewrite ? () => option.rewrite : pattern => path => path.replace(pattern, '');
    }
  }
  // handle event：proxyReq、proxyRes、error
  handle(proxyServer) {
    const { handleReq, handleRes, handleError } = this.options;
    proxyServer.on('proxyReq', (proxyReq, req, res, options) => {
      if (handleReq) {
        handleReq.call(null, { proxyReq, req, res, options });
      }
      const contentType = proxyReq.getHeader('Content-Type');
      if (req.body && contentType) {
        let bodyData;
        if (contentType.match(/application\/json/)) {
          bodyData = JSON.stringify(req.body);
        } else if (contentType.match(/application\/x-www-form-urlencoded/)) {
          bodyData = queryString.stringify(req.body);
        }
        if (bodyData) {
          proxyReq.write(bodyData);
          proxyReq.end();
        }
      }
    });
    proxyServer.on('proxyRes', (proxyRes, req, res) => {
      if (handleRes) {
        handleRes.call(null, { proxyRes, req, res });
      }
    });
    proxyServer.on('error', (err, req, res) => {
      if (handleError) {
        handleError.call(null, { err, req, res });
      }
    });
  }
}

module.exports = baseProxy;
